@@ -7,10 +7,12 @@
 
 use std::collections::{HashMap, HashSet};
 use std::fmt;
+use std::fs;
 use std::mem;
 
 use futures::future::join_all;
 use futures::Future;
+use indexmap::{IndexMap, IndexSet};
 use tokio_core::reactor::Core;
 
 extern crate itertools;
@@ -35,6 +37,8 @@ pub struct DependencyResolver<'ctx> {
     table: HashMap<&'ctx str, Dependency<'ctx>>,
     /// A cache of decisions made by the user during the resolution.
     decisions: HashMap<&'ctx str, usize>,
+    /// Checkout Directory overrides in case checkout_dir is defined and contains folders.
+    checked_out: HashMap<String, config::Dependency>,
 }
 
 impl<'ctx> DependencyResolver<'ctx> {
@@ -45,6 +49,7 @@ impl<'ctx> DependencyResolver<'ctx> {
             sess: sess,
             table: HashMap::new(),
             decisions: HashMap::new(),
+            checked_out: HashMap::new(),
         }
     }
 
@@ -62,6 +67,28 @@ impl<'ctx> DependencyResolver<'ctx> {
         let mut core = Core::new().unwrap();
         let io = SessionIo::new(self.sess, core.handle());
 
+        // Store path dependencies already in checkout_dir
+        match self.sess.manifest.workspace.checkout_dir.clone() {
+            Some(checkout) => {
+                if checkout.exists() {
+                    for dir in fs::read_dir(checkout).unwrap() {
+                        self.checked_out.insert(
+                            dir.as_ref()
+                                .unwrap()
+                                .path()
+                                .file_name()
+                                .unwrap()
+                                .to_str()
+                                .unwrap()
+                                .to_string(),
+                            config::Dependency::Path(dir.unwrap().path()),
+                        );
+                    }
+                }
+            }
+            None => {}
+        }
+
         // Load the plugin dependencies.
         self.register_dependencies_in_manifest(
             &self.sess.config.plugins,
@@ -190,6 +217,7 @@ impl<'ctx> DependencyResolver<'ctx> {
             .iter()
             .map(|(name, dep)| {
                 let name = name.as_str();
+                let dep = self.checked_out.get(name).unwrap_or(dep);
                 let dep = self.sess.config.overrides.get(name).unwrap_or(dep);
                 (name, self.sess.load_dependency(name, dep, manifest))
             })
@@ -271,6 +299,9 @@ impl<'ctx> DependencyResolver<'ctx> {
                     let pkg_name = self.sess.intern_string(m.package.name.clone());
                     m.dependencies.iter().map(move |(n, d)| (n, (pkg_name, d)))
                 })
+                .map(|(name, (pkg_name, dep))| {
+                    (name, (pkg_name, self.checked_out.get(name).unwrap_or(dep)))
+                })
                 .map(|(name, (pkg_name, dep))| {
                     (
                         name,
@@ -322,7 +353,7 @@ impl<'ctx> DependencyResolver<'ctx> {
         name: &str,
         con: &DependencyConstraint,
         src: &DependencySource<'ctx>,
-    ) -> Result<Option<std::collections::HashSet<usize>>> {
+    ) -> Result<Option<indexmap::IndexSet<usize>>> {
         use self::DependencyConstraint as DepCon;
         use self::DependencyVersions as DepVer;
         match (con, &src.versions) {
@@ -335,10 +366,24 @@ impl<'ctx> DependencyResolver<'ctx> {
                     .enumerate()
                     .map(|(id, &hash)| (hash, id))
                     .collect();
-                let revs: HashSet<usize> = gv
+                let mut revs_tmp: IndexMap<_, _> = gv
                     .versions
                     .iter()
-                    .filter_map(|&(ref v, h)| {
+                    .sorted()
+                    .filter_map(
+                        |&(ref v, h)| {
+                            if con.matches(v) {
+                                Some((v, h))
+                            } else {
+                                None
+                            }
+                        },
+                    )
+                    .collect();
+                revs_tmp.reverse();
+                let revs: IndexSet<usize> = revs_tmp
+                    .iter()
+                    .filter_map(|(ref v, h)| {
                         if con.matches(v) {
                             Some(hash_ids[h])
                         } else {
@@ -351,7 +396,7 @@ impl<'ctx> DependencyResolver<'ctx> {
             }
             (&DepCon::Revision(ref con), &DepVer::Git(ref gv)) => {
                 // TODO: Move this outside somewhere. Very inefficient!
-                let revs: HashSet<usize> = gv
+                let mut revs: IndexSet<usize> = gv
                     .refs
                     .get(con.as_str())
                     .map(|rf| {
@@ -370,6 +415,7 @@ impl<'ctx> DependencyResolver<'ctx> {
                             )
                             .collect()
                     });
+                revs.sort();
                 // debugln!("resolve: `{}` matches revision `{}` for revs {:?}", name, con, revs);
                 Ok(Some(revs))
             }
@@ -423,7 +469,7 @@ impl<'ctx> DependencyResolver<'ctx> {
         // debugln!("resolve: restricting `{}` to versions {:?}", name, indices);
 
         if indices.is_empty() {
-            core.run(io.dependency_versions(src.id, true))?;
+            src.versions = core.run(io.dependency_versions(src.id, true))?;
 
             let indices = match self.req_indices(name, con, src) {
                 Ok(o) => match o {
@@ -447,10 +493,10 @@ impl<'ctx> DependencyResolver<'ctx> {
             State::Open => unreachable!(),
             State::Locked(_) => unreachable!(), // TODO: This needs to do something.
             State::Constrained(ref ids) | State::Picked(_, ref ids) => {
-                let is_ids = (*ids)
-                    .intersection(&indices)
+                let is_ids = indices
+                    .intersection(&ids)
                     .map(|i| *i)
-                    .collect::<HashSet<usize>>();
+                    .collect::<IndexSet<usize>>();
                 if is_ids.is_empty() {
                     let mut msg = format!(
                         "Requirement `{}` conflicts with other requirements on dependency `{}`.\n",
@@ -547,11 +593,11 @@ impl<'ctx> DependencyResolver<'ctx> {
                                     dep.name,
                                     src.id
                                 );
-                                State::Picked(0, HashSet::new())
+                                State::Picked(0, IndexSet::new())
                             }
                             DependencyVersions::Git(..) => {
                                 debugln!("resolve: picking version for `{}[{}]`", dep.name, src.id);
-                                State::Picked(ids.iter().map(|i| *i).min().unwrap(), ids.clone())
+                                State::Picked(ids.first().map(|i| *i).unwrap(), ids.clone())
                             }
                             DependencyVersions::Registry(..) => {
                                 return Err(Error::new(format!("Version picking for registry dependency `{}` not yet imlemented", dep.name)));
@@ -738,9 +784,9 @@ enum State {
     /// The dependency has been locked in the lockfile.
     Locked(usize),
     /// The dependency may assume any of the listed versions.
-    Constrained(HashSet<usize>),
+    Constrained(IndexSet<usize>),
     /// The dependency had a version picked.
-    Picked(usize, HashSet<usize>),
+    Picked(usize, IndexSet<usize>),
 }
 
 impl State {
