@@ -129,12 +129,7 @@ impl<'sess, 'ctx: 'sess> Session<'ctx> {
             cfg,
             manifest.package.name
         );
-        let src = match *cfg {
-            config::Dependency::Version(_) => DependencySource::Registry,
-            config::Dependency::Path(ref p) => DependencySource::Path(p.clone()),
-            config::Dependency::GitRevision(ref g, _)
-            | config::Dependency::GitVersion(ref g, _) => DependencySource::Git(g.clone()),
-        };
+        let src = DependencySource::from(cfg);
         self.deps
             .lock()
             .unwrap()
@@ -495,6 +490,7 @@ impl<'io, 'sess: 'io, 'ctx: 'sess> SessionIo<'sess, 'ctx> {
         let name2 = String::from(name);
         let url = String::from(url);
         let url2 = url.clone();
+        let url3 = url.clone();
 
         // Either initialize the repository or update it if needed.
         if !db_dir.join("config").exists() {
@@ -518,6 +514,12 @@ impl<'io, 'sess: 'io, 'ctx: 'sess> SessionIo<'sess, 'ctx> {
                     })
                     .and_then(move |_| git.fetch("origin"))
                     .map_err(move |cause| {
+                        if url3.contains("git@") {
+                            warnln!(
+                                "Please ensure your public ssh key is added to the git server."
+                            );
+                        }
+                        warnln!("Please ensure the url is correct and you have access to the repository.");
                         Error::chain(
                             format!("Failed to initialize git database in {:?}.", db_dir),
                             cause,
@@ -542,6 +544,12 @@ impl<'io, 'sess: 'io, 'ctx: 'sess> SessionIo<'sess, 'ctx> {
                     })
                     .and_then(move |_| git.fetch("origin"))
                     .map_err(move |cause| {
+                        if url3.contains("git@") {
+                            warnln!(
+                                "Please ensure your public ssh key is added to the git server."
+                            );
+                        }
+                        warnln!("Please ensure the url is correct and you have access to the repository.");
                         Error::chain(
                             format!("Failed to update git database in {:?}.", db_dir),
                             cause,
@@ -1370,6 +1378,17 @@ pub enum DependencySource {
     Git(String),
 }
 
+impl<'a> From<&'a config::Dependency> for DependencySource {
+    fn from(cfg: &'a config::Dependency) -> DependencySource {
+        match *cfg {
+            config::Dependency::Path(ref path) => DependencySource::Path(path.clone()),
+            config::Dependency::GitRevision(ref url, _) => DependencySource::Git(url.clone()),
+            config::Dependency::GitVersion(ref url, _) => DependencySource::Git(url.clone()),
+            config::Dependency::Version(_) => DependencySource::Registry,
+        }
+    }
+}
+
 impl fmt::Display for DependencySource {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
